#r "System.Windows.Forms"

using System.Windows.Forms;

/// <summary>
/// 線の始点、終点がともに一致するか
/// </summary>
/// <returns></returns>
/// <remarks>
/// </remarks> 
bool IsStartEndEquals(LineShape line1, LineShape line2)
{
    // line1 と line2 の始点終点が同じか
    return ((line1.StartPoint.Equals(line2.StartPoint) || line1.StartPoint.Equals(line2.EndPoint)) &&
       (line1.EndPoint.Equals(line2.StartPoint) || line1.EndPoint.Equals(line2.EndPoint))) ? true : false;

}

/// <summary>
/// 線のプロパティが一致するか
/// </summary>
/// <returns></returns>
/// <remarks>
/// </remarks> 
bool IsPropatiesEquals(LineShape line1, LineShape line2)
{
    return (line1.Drawing == line2.Drawing &&
            line1.ColorNumber == line2.ColorNumber &&
            line1.LinetypeNumber == line2.LinetypeNumber &&
            line1.LinewidthNumber == line2.LinewidthNumber) ? true : false;
}


// 選択されている図形を取得する
var selectedShapes = Application.ActiveDocument.SelectionManager.SelectedShapes;

// 選択数をチェック
if(selectedShapes.Count > 1000)
{
    MessageBox.Show("選択図形が 1000件を超えています。\n図形を絞り込んで 1000件以下にしてください。");
    return;
}

// 線分図形のリスト
var lineShapes = new List<LineShape>();

// 選択図形から線分のみ抽出する
foreach(SelectedShape selectedShape in selectedShapes)
{
    // 線分の場合
    if(selectedShape.Shape is LineShape)
        lineShapes.Add((LineShape)selectedShape.Shape);
}


// 削除する図形のリスト
var delLines = new List<LineShape>();

// 選択図形のループ
foreach(LineShape line1 in lineShapes)
{
    // 編集可能かチェック
    if(line1.Layer.Editable != true ||
        line1.Drawing.Editable != true)
        continue;

    // 比較図形の始点と終点を取得する
    var startPoint = line1.StartPoint;
    var endPoint = line1.EndPoint;

    bool bDel = false;
    // 選択図形中の線分と比較するためのループ
    foreach(LineShape line2 in lineShapes)
    {
        if(line1.RecordID == line2.RecordID)
            continue;

        // 同じ部分図
        // 同じ色、線種、線幅
        // のみ処理をする
        if(!IsPropatiesEquals(line1, line2))
            continue;

        // line1 が line2 に完全に含まれる場合
        if(line2.IsPointOnShape(startPoint) &&
            line2.IsPointOnShape(endPoint))
        {
            // 削除対象とする
            bDel = true;
            break;
        }
    }

    if(bDel)
        // 削除リストに追加する
        delLines.Add(line1);
}

// 削除除外する図形のリスト
var excludeLines = new List<LineShape>();

// 削除対象のチェック
// 始点、終点が同じ線分は一方を残す
// ただし、その線がさらに大きな線分に完全に含まれている場合は残さない
foreach(LineShape delLine1 in delLines)
{
    bool bContinue = false;

    // 除外リストを検索
    foreach(LineShape exLine in excludeLines)
    {
        // 同じ始点終点の図形がすでに除外リストにある
        if(IsStartEndEquals(delLine1, exLine))
        {
            bContinue = true;
            break;
        }
    }
    if(bContinue)
        continue;

    foreach(LineShape delLine2 in delLines)
    {
        if(delLine1.RecordID == delLine2.RecordID)
            continue;

        // delLine1 と delLine2 の始点終点が同じ場合
        if(IsStartEndEquals(delLine1, delLine2))
        {
            bool bExclude = true;   // 除外するか
            
            // 選択された線図形に完全に含まれるかチェック
            foreach(LineShape selectedLine in lineShapes)
            {
                if(delLine1.RecordID == selectedLine.RecordID)
                    continue;

                // 同じ部分図
                // 同じ色、線種、線幅
                // のみ処理をする
                if(!IsPropatiesEquals(delLine1, selectedLine))
                    continue;

                // delLine1 が selectedLine に完全に含まれる場合
                if(selectedLine.IsPointOnShape(delLine1.StartPoint) &&
                    selectedLine.IsPointOnShape(delLine1.EndPoint))
                {
                    // 始点終点が一致しない場合
                    if(!IsStartEndEquals(delLine1, selectedLine))
                    {
                        // 除外しない（削除対象とする）
                        bExclude = false;
                        break;
                    }
                }
            }
            if(bExclude)
            {
                excludeLines.Add(delLine1);
                break;
            }
        }
    }
}

int delCount = delLines.Count - excludeLines.Count;
if(delCount > 0)
{
    var result = MessageBox.Show(
            delCount.ToString() + " 件の図形を削除します。\nよろしいですか？",
            "重複線の削除",
            MessageBoxButtons.YesNo);

    if(result == DialogResult.Yes)
    {
        var doc = Application.ActiveDocument;

        // UNDO 処理を開始
        doc.UndoManager.BeginUndoUnit();

        // 選択を解除
        doc.SelectionManager.ClearSelection();

        // 削除を実行
        foreach(LineShape shape in delLines)
        {
            // 除外リストに含まれていない場合は削除
            if(!excludeLines.Contains(shape))
                shape.Drawing.Shapes.Remove(shape);
        }

        // UNDO 処理を終了
        doc.UndoManager.EndUndoUnit();

        MessageBox.Show("削除しました。");
    }
}
else
{
    MessageBox.Show("選択中の図形に重複線は見つかりませんでした。");
}
